/*
*  Power BI Visual CLI
*
*  Copyright (c) Microsoft Corporation
*  All rights reserved.
*  MIT License
*
*  Permission is hereby granted, free of charge, to any person obtaining a copy
*  of this software and associated documentation files (the ""Software""), to deal
*  in the Software without restriction, including without limitation the rights
*  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
*  copies of the Software, and to permit persons to whom the Software is
*  furnished to do so, subject to the following conditions:
*
*  The above copyright notice and this permission notice shall be included in
*  all copies or substantial portions of the Software.
*
*  THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
*  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
*  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
*  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
*  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
*  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
*  THE SOFTWARE.
*/
"use strict";

import "core-js/stable";
import "../../style/visual.less";
import powerbi from "powerbi-visuals-api";
import VisualConstructorOptions = powerbi.extensibility.visual.VisualConstructorOptions;
import VisualUpdateOptions = powerbi.extensibility.visual.VisualUpdateOptions;
import IVisual = powerbi.extensibility.visual.IVisual;
import EnumerateVisualObjectInstancesOptions = powerbi.EnumerateVisualObjectInstancesOptions;
import VisualObjectInstance = powerbi.VisualObjectInstance;
import DataView = powerbi.DataView;
import VisualObjectInstanceEnumerationObject = powerbi.VisualObjectInstanceEnumerationObject;
import IVisualEventService = powerbi.extensibility.IVisualEventService;
import * as visualInterfaces from "./visualInterfaces";
import { VisualSettings } from "../settings/visualSettings";
import {filterManager,managerFactory,selectorManager,layoutManager} from "../managers";
import * as d3 from "d3";
import { getObject } from "powerbi-visuals-utils-dataviewutils/lib/dataViewObjects";
import { getMeasureIndexOfRole } from "powerbi-visuals-utils-dataviewutils/lib/dataRoleHelper";
import { selector } from "d3";

export class Visual implements IVisual {
    private events: IVisualEventService;
    private settings: VisualSettings;
    private layoutManager: visualInterfaces.ILayoutManager;
    private selectorManager: visualInterfaces.ISelectorManager;
    private view:SVGElement;
    private filterManager:visualInterfaces.IFilterManager;

    constructor(options: VisualConstructorOptions) {
        console.log('Visual constructor', options);
        this.events = options.host.eventService;
        if (document) {
            let container=d3.select(options.element).append("div").classed("container",true);
            let headerContainer=container.append("div").classed("header-container",true);
            let selectorContainer=container.append("div").classed("selector-container",true);
            this.layoutManager=managerFactory.CreateLayoutManager(layoutManager,container);
            this.selectorManager=managerFactory.CreateSelectorManager(selectorManager,selectorContainer);
            this.filterManager=managerFactory.CreateFilterManager(filterManager,options.host);
        }
        console.debug('end constructor');
    }

    public update(options: VisualUpdateOptions) {
        console.debug('visual update start');
        this.events.renderingStarted(options);
        
        this.settings = Visual.parseSettings(options && options.dataViews && options.dataViews[0]);
        let dataView=options.dataViews[0];
        let field=dataView.categorical.categories[0];
        let values=dataView.categorical.values;
        let defaultSelect:powerbi.DataViewValueColumn;
        let defaultStart:powerbi.DataViewValueColumn;
        let defaultEnd:powerbi.DataViewValueColumn;
        values.map((messure)=>{
            var role=messure.source.roles;
            if(role['defaultSelect']){
                defaultSelect=messure;
            }else if(role['defaultStart']){
                defaultStart=messure;
            }else if(role['defaultEnd']){
                defaultEnd=messure;
            };
        });
        this.layoutManager.update(dataView);
        this.selectorManager.updateView(field,defaultSelect,defaultStart,defaultEnd);
        this.events.renderingFinished(options);
    }

    private static parseSettings(dataView: DataView): VisualSettings {
        return <VisualSettings>VisualSettings.parse(dataView);
    }

    /**
     * This function gets called for each of the objects defined in the capabilities files and allows you to select which of the
     * objects and properties you want to expose to the users in the property pane.
     *
     */
    public enumerateObjectInstances(options: EnumerateVisualObjectInstancesOptions): VisualObjectInstance[] | VisualObjectInstanceEnumerationObject {
        return VisualSettings.enumerateObjectInstances(this.settings || VisualSettings.getDefault(), options);
    }
}